#include "library.h"

void System_Init()
{
	/* Reset SW, HPRE, PPRE1, PPRE2, ADCPRE and MCO bits */
	RCC->CFGR &= (u32)0xF8FF0000;
}

void assert_failed(char* file, unsigned long line)
{
	while(1);
}

void delay(unsigned long ms)
{
	unsigned long i;
	for (i=0; i<3250*ms; i++);
}

void PORT_Init()
{
	GPIO_InitTypeDef GPIO_InitStructure;

	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOE, ENABLE);

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 |
		GPIO_Pin_2 | GPIO_Pin_3 | GPIO_Pin_4 | GPIO_Pin_5 |
		GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_10 | GPIO_Pin_11 |
		GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_10MHz;
	GPIO_Init(GPIOE, &GPIO_InitStructure);

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8 | GPIO_Pin_9;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPD;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_10MHz;
	GPIO_Init(GPIOE, &GPIO_InitStructure);
}

char PORT_ReadInputPin(unsigned short pin)
{
	return (GPIOE->IDR & pin) ? 1 : 0;
}

unsigned short PORT_ReadInput()
{
	return GPIOE->IDR;
}

unsigned short PORT_ReadOutput()
{
	return GPIOE->ODR;
}

void PORT_Write(unsigned short value)
{
	GPIOE->ODR = value;
}

void TIMER_Init(TIM_TypeDef *timer)
{
	NVIC_InitTypeDef NVIC_Config;
	char TIM_Ok=0;

	if (timer == TIM2)
	{
		TIM_Ok = 1;
		RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);
		NVIC_Config.NVIC_IRQChannel = TIM2_IRQChannel;
	}

	if (timer == TIM3)
	{
		TIM_Ok = 1;
		RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE);
		NVIC_Config.NVIC_IRQChannel = TIM3_IRQChannel;
	}

	if (timer == TIM4)
	{
		TIM_Ok = 1;
		RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM4, ENABLE);
		NVIC_Config.NVIC_IRQChannel = TIM4_IRQChannel;
	}

	if (!TIM_Ok) return;

	NVIC_Config.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_Config.NVIC_IRQChannelSubPriority = 0;
	NVIC_Config.NVIC_IRQChannelCmd = ENABLE;

	NVIC_Init(&NVIC_Config);
}

void TIMER_Start(TIM_TypeDef *timer, unsigned int prescaler, unsigned int counter)
{
	TIM_TimeBaseInitTypeDef TIM_Config;

	TIM_Config.TIM_Period = counter;
	TIM_Config.TIM_Prescaler = prescaler;
	TIM_Config.TIM_ClockDivision = TIM_CKD_DIV2;
	TIM_Config.TIM_CounterMode = TIM_CounterMode_Down;

	TIM_TimeBaseInit(timer, &TIM_Config);
	TIM_ITConfig(timer, TIM_IT_Update, ENABLE);
	TIM_Cmd(timer, ENABLE);
}

void TIMER_Stop(TIM_TypeDef *timer)
{
	TIM_Cmd(timer, DISABLE);
}

void ANALOG_Init()
{
	GPIO_InitTypeDef GPIO_InitStructure;
	ADC_InitTypeDef ADC_InitStructure;

	RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE);
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 |
		GPIO_Pin_2 | GPIO_Pin_3;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	ADC_InitStructure.ADC_Mode = ADC_Mode_Independent;
	ADC_InitStructure.ADC_ScanConvMode = DISABLE;
	ADC_InitStructure.ADC_ContinuousConvMode = DISABLE;
	ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_None;
	ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
	ADC_InitStructure.ADC_NbrOfChannel = 1;
	ADC_Init(ADC1, &ADC_InitStructure);

	ADC_RegularChannelConfig(ADC1, ADC_Channel_0, 1, ADC_SampleTime_239Cycles5);

	ADC_Cmd(ADC1, ENABLE);

	ADC_ResetCalibration(ADC1);
	while (ADC_GetResetCalibrationStatus(ADC1));
	ADC_StartCalibration(ADC1);
	while (ADC_GetCalibrationStatus(ADC1));
}

unsigned int ANALOG_Read()
{
	ADC_SoftwareStartConvCmd(ADC1, ENABLE);
	while (!ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC));
	ADC_ClearFlag(ADC1, ADC_FLAG_EOC);

	return ADC_GetConversionValue(ADC1);
}
